@@ -24,6 +24,9 @@ pika_targets=(
 "task_latency_test"
 "task_latency_test"
 "task_latency_test"
+"task_yield_test"
+"task_yield_test"
+"condition_variable_overhead_test"
 )
 pika_test_options=(
 "--pika:ini=pika.thread_queue.init_threads_count=100 \
@@ -66,6 +69,22 @@ pika_test_options=(
 --nostack
 --pika:threads=2
 --perftest-json"
+
+"--repetitions=100
+--num-yields=100000
+--pika:threads=1
+--perftest-json"
+
+"--repetitions=100
+--num-yields=100000
+--pika:threads=2
+--perftest-json"
+
+"--loops=1000000
+--repetitions=3
+--pika:threads=2
+--perftest-json"
+
 )
 
 index=0
