@@ -345,7 +345,7 @@ int pika_main(pika::program_options::variables_map& vm)
     pika::threads::remove_scheduler_mode(::pika::threads::scheduler_mode::enable_idle_backoff);
 
     pika::util::mpi_environment mpi_env;
-    pika::runtime* rt = pika::get_runtime_ptr();
+    pika::detail::runtime* rt = pika::detail::get_runtime_ptr();
     pika::util::runtime_configuration cfg = rt->get_config();
     mpi_env.init(nullptr, nullptr, cfg);
 
