@@ -394,6 +394,21 @@ namespace pika::detail {
         }
     }
 
+#if defined(PIKA_HAVE_MPI)
+    std::size_t handle_mpi_completion_mode(
+        detail::manage_config& cfgmap, pika::program_options::variables_map& vm)
+    {
+        std::size_t completion_mode = cfgmap.get_value<std::size_t>("pika.mpi.completion_mode", 0);
+
+        if (vm.count("pika:mpi-completion-mode"))
+        {
+            completion_mode = vm["pika:mpi-completion-mode"].as<std::size_t>();
+        }
+
+        return completion_mode;
+    }
+#endif
+
     ///////////////////////////////////////////////////////////////////////////
     void command_line_handling::handle_arguments(detail::manage_config& cfgmap,
         pika::program_options::variables_map& vm, std::vector<std::string>& ini_config)
@@ -532,6 +547,11 @@ namespace pika::detail {
                 std::to_string(num_high_priority_queues));
         }
 
+#if defined(PIKA_HAVE_MPI)
+        std::size_t const mpi_completion_mode = detail::handle_mpi_completion_mode(cfgmap, vm);
+        ini_config.emplace_back("pika.mpi.completion_mode=" + std::to_string(mpi_completion_mode));
+#endif
+
         update_logging_settings(vm, ini_config);
 
         if (debug_clp)
